-- HazLoot library containing the data for a classification list

HazLootList = {}
HazLootList.ItemCount = 0
HazLootList.RoleByItem = {} -- Database of items
-- Why aren't these bitmaps? Because bitmaps aren't native to Lua, this is
-- just as (in)efficient as using the bitmap libraries, and probably easier
-- for laypeople to modify.
HazLootList.UNCLASSIFIED = 0
HazLootList.ROLES = { "Random roll",  -- 100
                      "Tier piece",   -- 200
                      "Caster DPS",   -- 300
                      "Healer",       -- 400
                      "Physical DPS", -- 500
                      "Tank",         -- 600
}
HazLootList.ROLES[0] = "<unclassified>"
HazLootList.ROLEVALUE = { ["<unclassified>"]=0,
                          ["Random roll"]=100,
                          ["Tier piece"]=200,
                          ["Caster DPS"]=300,
                          ["Healer"]=400,
                          ["Physical DPS"]=500,
                          ["Tank"]=600,
}
HazLootList.ANYRANDOM = HazLootList.ROLES["Random roll"]
HazLootList.RESTRICTIONS = { -- "Any" is a special case: 0
                         "Death Knight", --1
                         "Druid",        --2
                         "Hunter",       --3
                         "Mage",         --4
                         "Paladin",      --5
                         "Priest",       --6
                         "Rogue",        --7
                         "Shaman",       --8
                         "Warlock",      --9
                         "Warrior",      --10
}
HazLootList.RESTVALUES = {
                         ["Death Knight"]=1,
                         ["Druid"]=2,
                         ["Hunter"]=3,
                         ["Mage"]=4,
                         ["Paladin"]=5,
                         ["Priest"]=6,
                         ["Rogue"]=7,
                         ["Shaman"]=8,
                         ["Warlock"]=9,
                         ["Warrior"]=10,
}
HazLootList.RESTRICTIONS[0] = "<unclassified>"

-- itemRoleString:
-- itemName [ | role[: restriction] [, role[: restriction] ] ]
-- if no role is specified, the default, "Random roll" is used
-- e.g., the following are all valid:
--   Rod of Blasting
--   Rod of Lordly Might|Tank,Physical DPS:Paladin
--   Wand of Fireball|Caster DPS
--   Phylactery of Pain|Caster DPS,Healer
--   Codpiece of the Lost Wastrel|Tier piece:Hunter,Tier piece:Shaman,Tier piece:Warrior
-- the following are NOT valid and will be ignored:
--   Shield of This|Wall of That|Tank
--   Mace of Hitting,Physical DPS
--   Codpiece of the Lost Cad|Tier piece:Druid,Rogue,Paladin
function HazLootList:AddItem(itemRoleString)
  local sep = string.find(itemRoleString,"|")
  if nil == sep then -- Just an item name, any random
    HazLootList.RoleByItem[itemRoleString] = {HazLootList.ANYRANDOM}
    HazLootList.ItemCount = HazLootList.ItemCount + 1
    return itemRoleString
  end
  local itemName  = string.sub(itemRoleString, 1, sep - 1)
  HazLootList.RoleByItem[itemName] = {}
  local roles = string.sub(itemRoleString, sep + 1 )
  local restRole, restrict
  local i = 0
  local roleValue
  for roleName in string.gmatch(roles,"([^,]+),?") do
    i = i + 1
    if nil == roleName then
      return "nil - a problem"
    end
    -- is there a restriction?
    sep = string.find(roleName,":")
    if nil == sep then -- No
      HazLootList.RoleByItem[itemName][i]=HazLootList:GetRoleValue(roleName)
    else -- yes, what is it?
      restRole   = string.sub(roleName, 1, sep - 1)
      restrict   = string.sub(roleName, sep + 1 )
      HazLootList.RoleByItem[itemName][i] = HazLootList:GetRoleValue(restRole) + HazLootList:GetRestriction(restrict)
    end
  end
  HazLootList.ItemCount = HazLootList.ItemCount + 1
end

-- Add an "Unclassified" entry for this item
function HazLootList:AddUnclassifiedtem(itemName)
  HazLootList.RoleByItem[itemName] = {HazLootList.UNCLASSIFED}
  HazLootList.ItemCount = HazLootList.ItemCount + 1
end

function HazLootList:GetRoleValue(roleName)
  local roleValue = HazLootList.ROLEVALUE[roleName]
  if nil == roleValue then -- invalid input
    -- TODO: make this create a new classification?
    return "nil - a problem"
  end
  return roleValue
end

function HazLootList:GetRestriction(restrict)
  local rValue = HazLootList.RESTVALUES[restrict]
  if nil == rValue then -- invalid input
    -- TODO: make this create a new classification?
    return "nil - a problem"
  end
  return rValue
end

-- Lazily parse the item string for the role table
function HazLootList:GetRoleTable(itemName)
  if not HazLootList.RoleByItem[itemName] then
    local lootString = TheLoot:GetLootString(itemName)
    if lootString then
      HazLootList:AddItem(lootString)
    else
      HazLootList:AddUnclassifiedtem(itemName)
    end
  end
  return HazLootList.RoleByItem[itemName]
end

-- TODO: convert to itemID for localization
function HazLootList:GetRolesOf(itemName)
  local roleTable = HazLootList.RoleByItem[itemName]
  local outString = ""
  if nil == roleTable then return -1 end
  for i,role in ipairs(roleTable) do
    if i > 1 then outString = outString .. ", " end
    itemRole, itemRestriction = HazLootList:GetRoleName(role)
    outString = outString .. itemRole .. (nil == itemRestriction and "" or ":"..itemRestriction)
  end
  return outString
end

function HazLootList:IsTierPiece(itemId)
  self:Debug("itemID: "..itemId)
  local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType,
        itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemId)
  local firstRole = HazLootList.RoleByItem[itemName]
  return HazLootList.ROLES["Tier piece"] == (role - role%100)
end

function HazLootList:GetRoleName(role)
  local restriction = role%100
  if 0 == restriction then return HazLootList.ROLES[role/100]
  elseif restriction > table.getn(HazLootList.RESTRICTIONS) then return HazLootList.RESTRICTIONS[0]
  end
  return HazLootList.ROLES[(role - restriction)/100], HazLootList.RESTRICTIONS[restriction]
end

function HazLootList:HowMuchLoot()
  return HazLootList.ItemCount
end

-- Sort Loot IDs: return true if loot1 should be sorted before loot2
function HazLootList:LootOrderComparator(itemId1,itemId2)
  if nil == itemId2 then return true end
  local itemId1value,itemId2value
  itemValue1 = HazLootList:GetLootValue(itemId1)
  itemValue2 = HazLootList:GetLootValue(itemId2)
  return (itemValue1 == itemValue2) and (itemId1 < itemId2) or (itemValue1 < itemValue2)
end

function HazLootList:GetLootValue(itemId)
  -- 1. Tier pieces
  if HazLootList:IsTierPiece(loot1)
  then return 1 end
  local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType,
        itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemId)
  -- 2. Trinkets, Rings, Necks
  if itemEquipLoc == INVTYPE_NECK
     or itemEquipLoc == INVTYPE_FINGER
     or itemEquipLoc == INVTYPE_TRINKET
  then return 2 end
  -- 3. Weapons, Shields, Wands, Relics, Idols
  if itemEquipLoc == INVTYPE_WEAPON
    or itemEquipLoc == INVTYPE_SHIELD
    or itemEquipLoc == INVTYPE_2HWEAPON
    or itemEquipLoc == INVTYPE_WEAPONMAINHAND
    or itemEquipLoc == INVTYPE_WEAPONOFFHAND
    or itemEquipLoc == INVTYPE_HOLDABLE
    or itemEquipLoc == INVTYPE_RANGED
    or itemEquipLoc == INVTYPE_THROWN
    or itemEquipLoc == INVTYPE_RANGEDRIGHT
    or itemEquipLoc == INVTYPE_RELIC
    or itemSubType  == "Miscellaneous"
  then return 3 end
  -- TODO: figure out how to localize this
  if itemType == "Armor" then
    if itemSubType == "Plate"
      then return 4 end
    if itemSubType == "Mail"
      then return 5 end
    if itemSubType == "Leather"
      then return 6 end
    if itemSubType == "Cloth"
      then return 7 end
  end
  return 0 -- unclassified
end

-------- (half-hearted) TESTING CODE --------
HazLootList.TestInput = {
"Any Healer|Healer",
"Druid Healer|Healer:Druid",
"DK Physical DPS and Warrior tank|Physical DPS:Death Knight,Tank:Warrior",
"Random roll|Random roll",
"Warrior Hunter Shaman tier piece|Tier piece:Warrior,Tier piece:Hunter,Tier piece:Shaman",
"unclassified|Some crap",
}
HazLootList.TestItems = {
  ["Any Healer"]={400},
  ["Druid Healer"]={402},
  ["DK Physical DPS, Warrior tank"]={501,610},
  ["Random roll"]={100},
  ["Warrior,Hunter,Shaman tier piece"]={203,208,210},
  ["unclassified"]={199},
}

function HazLootList:Test()
  print("*** TESTING GetRoleName ***")
  for item,roleTable in pairs(HazLootList.TestItems) do
    local itemRole, itemRestriction
    local outString = item.." has roles: \n\t"
    for i,role in ipairs(roleTable) do
      if i > 1 then outString = outString .. ", " end
      itemRole, itemRestriction = HazLootList:GetRoleName(role)
      outString = outString .. itemRole .. (nil == itemRestriction and "" or "("..itemRestriction..")")
    end
    print(outString)
  end
  print("*** TESTING AddItem ***")
  for i,input in ipairs(HazLootList.TestInput) do
    HazLootList:AddItem(input)
  end
  print("*** TESTING GetRolesOf ***")
end

-- HazLootList:Test()
